#!/system/bin/sh

# Set all output of this script to redirect to logcat
misty_log()
{
    log -p i -t "misty-init" "$@"
}

install_sensoryservices()
{
    if [ -e /data/misty/updateapk ]; then
        rm /data/misty/updateapk;
        if [ -e /sdcard/sensoryservices.apk ]; then
            misty_log "New sensory services apk found"

            misty_log "Installing sensory services"
            pm install -rg /sdcard/sensoryservices.apk
            sleep 1

            misty_log "Restarting sensory services"
            am start -n com.mistyrobotics.sensoryservices/com.mistyrobotics.sensoryservices.MainActivity
            sleep 1

            misty_processes=$(ps | grep misty)
            misty_log "$misty_processes"

            misty_log "Removing sensoryservices.apk"
            rm /sdcard/sensoryservices.apk

            misty_log "Sensory services update complete."
        else
            misty_log "No new apk file found"
        fi
    fi
}

setup_iptables()
{
    iptables -t nat -A POSTROUTING -o wlan0 -j MASQUERADE
    iptables -I FORWARD -i eth0 -o wlan0 -j ACCEPT
    iptables -I FORWARD -i wlan0 -o eth0 -m state --state RELATED,ESTABLISHED -j ACCEPT
    iptables -I FORWARD -i wlan0 -d 10.10.10.10 -p tcp -m multiport --dports 22,80,8080,443,445,8116,8117 -j ACCEPT
    iptables -t nat -I PREROUTING -i wlan0 -p tcp -m multiport --dports 22,80,8080,443,445,8116,8117 -j DNAT --to-destination 10.10.10.10
}

setup_eth0_address()
{
    # Set eth0 IP address when needed
    # Check the current hardware interface status, 0 is unplugged, 1 is plugged in
    status=$(cat /sys/class/net/eth0/carrier)
    # If we have an IP address on the interface, then we do not need to set it again
    ip_address=$(ifconfig eth0 | grep "inet addr" | cut -d : -f 2 | cut -d ' ' -f 1)
    if [ "$status" = 1 ] && [ -z "$ip_address" ]; then
        misty_log "eth0_status=$status ip_address=$ip_address"
        # Setup networking whenever the hardware interface is up and we do not have an IP address
        misty_log "Setup eth0 address"
        ip link set eth0 up
        ip addr add 10.10.10.100/24 dev eth0
        sleep 1
    fi
}

setup_eth0_table()
{
    # Check if the eth0 table exists
    # Android only uses this table as a backup for when wlan0 isn't present
    # Adding the route here enables intraSOM connectivity when there's no Internet
    if ip route show table eth0; then
        # Add eth0 route if not already present in eth0 table
        if ! ip route show table eth0 | grep -q 'eth0'; then
            misty_log "Setup eth0 route"
            ip route add 10.10.10.0/24 dev eth0 proto static scope link table eth0
        fi
    fi
}

setup_wlan0_table()
{
    # Check if the wlan0 table exists
    # If wlan0 table doesn't exist, then Android system manages the eth0 connection directly
    # through the ConnectivityService
    if ip route show table wlan0; then
        # Add eth0 route if not already present in wlan0 table
        if ! ip route show table wlan0 | grep -q 'eth0'; then
            misty_log "Setup ip rule"
            ip route add 10.10.10.0/24 dev eth0 proto static scope link table wlan0
            sleep 1
        fi
    fi
}

ping_410()
{
    # Check that we can ping the 410
    # If not, force a reset of eth0 settings
    ping_success=$(ping -c 1 10.10.10.10 | grep "packet loss" | cut -d , -f 2 | cut -c 2- | cut -c 1)
    if [ "$ping_success" = 0 ]; then
        misty_log "Ping failed. Reset eth0"
        ip link set eth0 down
        sleep 1
        ip link set eth0 up
    fi
}

capture_kernellog()
{
    if [ -f "/data/misty/capturebootlog" ]; then
        mkdir -p /data/misty/logs

        local log2="/data/misty/logs/bootlog.txt.2"
        if [ -f "$log2" ]; then
            rm $log2
        fi

        local log1="/data/misty/logs/bootlog.txt.1"
        if [ -f "$log1" ]; then
            mv $log1 $log2
        fi

        local log="/data/misty/logs/bootlog.txt"
        if [ -f "$log" ]; then
            mv $log $log2
        fi

        # Capture dmesg into a log file endlessly.
        misty_log "capturing dmesg to /data/misty/logs/bootlog.txt"
        nohup dmesg -w -r > $log &
    fi
}

capture_adblog()
{
    if [ -f "/data/misty/captureadblog" ]; then
        misty_log "capturing adb logs to /data/misty/logs/adblog.txt"
        mkdir -p /data/misty/logs
        nohup logcat -f /data/misty/logs/adblog.txt -r 100000 -n 10 &
    fi
}

init_tallylight()
{
    if [ ! -d "/sys/class/gpio/gpio123" ]; then
        misty_log "Tally light not initialized, trying again"
        echo 123 > /sys/class/gpio/export
        echo out > /sys/class/gpio/gpio123/direction
        chmod 777 /sys/class/gpio/gpio123/value
        echo 1 > /sys/class/gpio/gpio123/value
    fi
}

init_fanctrl()
{
    if [ ! -d "/sys/class/gpio/gpio135" ]; then
        misty_log "Fan control not initialized, trying again"
        echo 135 > /sys/class/gpio/export
        echo out > /sys/class/gpio/gpio135/direction
        chmod 777 /sys/class/gpio/gpio135/value
        echo 1 > /sys/class/gpio/gpio135/value
    fi
}

GOVERNOR_PERFORMANCE=0
set_governor()
{
    if [ -e /data/misty/performance ]; then
        if [ GOVERNOR_PERFORMANCE -eq 0 ]; then
            echo performance > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
            echo performance > /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor
            echo performance > /sys/devices/system/cpu/cpu2/cpufreq/scaling_governor
            echo performance > /sys/devices/system/cpu/cpu3/cpufreq/scaling_governor
            GOVERNOR_PERFORMANCE=1
        fi
    else
        if [ GOVERNOR_PERFORMANCE -eq 1 ]; then
            echo interactive > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
            echo interactive > /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor
            echo interactive > /sys/devices/system/cpu/cpu2/cpufreq/scaling_governor
            echo interactive > /sys/devices/system/cpu/cpu3/cpufreq/scaling_governor
            GOVERNOR_PERFORMANCE=0
        fi
    fi
}

# Main Execution
misty_log "Starting Misty configuration script"

# Ensure that IP Forward is turned on
echo 1 > /proc/sys/net/ipv4/ip_forward

# Turns off printk for improved SLAM performance
echo 0 > /proc/sys/kernel/printk

# Identify scripts version
touch /data/misty/scripts/scripts1.0.1

# Codec fix for audio recording failures after playing loud audio files.
echo 0x08a 0xf2 > /sys/kernel/debug/regmap/tasha-slim-pgd/registers
echo 0x614 0x39 > /sys/kernel/debug/regmap/tasha-slim-pgd/registers
echo 0x615 0x08 > /sys/kernel/debug/regmap/tasha-slim-pgd/registers
echo 0x624 0x00 > /sys/kernel/debug/regmap/tasha-slim-pgd/registers

capture_kernellog
capture_adblog

svc wifi enable
setup_iptables

while :
do
    init_tallylight

    init_fanctrl

    install_sensoryservices

    setup_eth0_address

    setup_eth0_table

    setup_wlan0_table

    ping_410

    set_governor

    sleep 5
done
